﻿using System.Collections;
using System.Collections.Specialized;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Markup;
using System.Windows.Media;

namespace WeakEventListenerDemo
{
    public class FixedControl : Control
    {
        WeakEventListener<FixedControl, object, NotifyCollectionChangedEventArgs> _weakEventListener;

        public FixedControl()
        {
            // Set a simple template/visuals
            Template = (ControlTemplate)XamlReader.Load(
                @"<ControlTemplate xmlns='http://schemas.microsoft.com/winfx/2006/xaml/presentation'>" +
                    @"<Grid Background='Green'><TextBlock Text='FixedControl' FontSize='32'/></Grid>" +
                @"</ControlTemplate>");
        }

        // Declare a standard ItemsControl.ItemsSource-like DependencyProperty
        public IEnumerable ItemsSource
        {
            get { return (IEnumerable)GetValue(ItemsSourceProperty); }
            set { SetValue(ItemsSourceProperty, value); }
        }
        public static readonly DependencyProperty ItemsSourceProperty = DependencyProperty.Register(
            "ItemsSource", typeof(IEnumerable), typeof(FixedControl), new PropertyMetadata(null, OnItemsSourceChanged));
        private static void OnItemsSourceChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((FixedControl)d).OnItemsSourceChanged((IEnumerable)e.OldValue, (IEnumerable)e.NewValue);
        }

        // Change handler for the ItemsControl.ItemsSource-like DependencyProperty
        private void OnItemsSourceChanged(IEnumerable oldValue, IEnumerable newValue)
        {
            // See if the old value implements INotifyCollectionChanged
            var oldNotifyCollectionChanged = oldValue as INotifyCollectionChanged;
            if (null != oldNotifyCollectionChanged)
            {
                // It does; detach our WeakEventListener and clear our reference
                _weakEventListener.Detach();
                _weakEventListener = null;
            }
            // See if the new value implements INotifyCollectionChanged
            var newNotifyCollectionChanged = newValue as INotifyCollectionChanged;
            if (null != newNotifyCollectionChanged)
            {
                // It does; create a WeakEventListener, attach us to it, and add it to the event
                _weakEventListener = new WeakEventListener<FixedControl, object, NotifyCollectionChangedEventArgs>(this);
                _weakEventListener.OnEventAction = (instance, source, eventArgs) =>
                    instance.OnCollectionChanged(source, eventArgs);
                _weakEventListener.OnDetachAction = (weakEventListener) =>
                    newNotifyCollectionChanged.CollectionChanged -= weakEventListener.OnEvent;
                newNotifyCollectionChanged.CollectionChanged += _weakEventListener.OnEvent;
            }
        }

        // Change handler for the INotifyCollectionChanged's CollectionChanged event
        private void OnCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            // Change handling code goes here...
        }
    }
}
