﻿using System.Collections;
using System.Collections.Specialized;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Markup;
using System.Windows.Media;

namespace WeakEventListenerDemo
{
    public class LeakyControl : Control
    {
        public LeakyControl()
        {
            // Set a simple template/visuals
            Template = (ControlTemplate)XamlReader.Load(
                @"<ControlTemplate xmlns='http://schemas.microsoft.com/winfx/2006/xaml/presentation'>" +
                    @"<Grid Background='Red'><TextBlock Text='LeakyControl' FontSize='32'/></Grid>" +
                @"</ControlTemplate>");
        }

        // Declare a standard ItemsControl.ItemsSource-like DependencyProperty
        public IEnumerable ItemsSource
        {
            get { return (IEnumerable)GetValue(ItemsSourceProperty); }
            set { SetValue(ItemsSourceProperty, value); }
        }
        public static readonly DependencyProperty ItemsSourceProperty = DependencyProperty.Register(
            "ItemsSource", typeof(IEnumerable), typeof(LeakyControl), new PropertyMetadata(null, OnItemsSourceChanged));
        private static void OnItemsSourceChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((LeakyControl)d).OnItemsSourceChanged((IEnumerable)e.OldValue, (IEnumerable)e.NewValue);
        }

        // Change handler for the ItemsControl.ItemsSource-like DependencyProperty
        private void OnItemsSourceChanged(IEnumerable oldValue, IEnumerable newValue)
        {
            // See if the old value implements INotifyCollectionChanged
            var oldNotifyCollectionChanged = oldValue as INotifyCollectionChanged;
            if (null != oldNotifyCollectionChanged)
            {
                // It does; detach from the CollectionChanged event
                oldNotifyCollectionChanged.CollectionChanged -= OnCollectionChanged;
            }
            // See if the new value implements INotifyCollectionChanged
            var newNotifyCollectionChanged = newValue as INotifyCollectionChanged;
            if (null != newNotifyCollectionChanged)
            {
                // It does; attach to the CollectionChanged event
                newNotifyCollectionChanged.CollectionChanged += OnCollectionChanged;
            }
        }

        // Change handler for the INotifyCollectionChanged's CollectionChanged event
        private void OnCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            // Change handling code goes here...
        }
    }
}
